import { world, EquipmentSlot, ItemStack, Player, BlockPermutation, Block, system, GameMode} from '@minecraft/server';

let eb;
let ei;

const crop = {
  onRandomTick({ block }) {
    const cropState = block.permutation.getState("bd:state");
    if (block.hasTag('bd:statet') && block.permutation.getState("bd:state") == 3) return;
    if (block.hasTag('bd:statef') && block.permutation.getState("bd:state") == 4) return;
    block.setPermutation(block.permutation.withState("bd:state", cropState + 1));
  },
  onPlayerInteract({ block, dimension, player }) {
    
    const cropState = block.permutation.getState("bd:state");
    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;
    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);

    if (block.permutation.getState("bd:state") == 4) {
      if (block.typeId == "bd:tomato_crop")
        dimension.spawnItem(new ItemStack("bd:tomato", Math.floor(Math.random() * 4) + 1), block.center());
      if (block.typeId == "bd:chilies_crop")
        dimension.spawnItem(new ItemStack("bd:chilies", Math.floor(Math.random() * 4) + 1), block.center());
      block.setPermutation(block.permutation.withState("bd:state", cropState - 2));
      dimension.playSound("block.sweet_berry_bush.pick", block.center());
      return;
    }
    if (mainhand.hasItem() && mainhand.typeId == "minecraft:bone_meal") {
      if (player.getGameMode() === GameMode.survival) {
        if (block.hasTag('bd:statet') && block.permutation.getState("bd:state") == 3) return;
        if (block.hasTag('bd:statef') && block.permutation.getState("bd:state") == 4) return;
        block.setPermutation(block.permutation.withState("bd:state", cropState + 1));
        dimension.playSound("item.bone_meal.use", block.center());
        dimension.spawnParticle("minecraft:crop_growth_emitter", block.center());
        if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
        else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
      }
      else {
        if (block.hasTag('bd:statet') && block.permutation.getState("bd:state") == 3) return;
        if (block.hasTag('bd:statef') && block.permutation.getState("bd:state") == 4) return;
        if (block.hasTag('bd:statet')) block.setPermutation(block.permutation.withState("bd:state", 3));
        if (block.hasTag('bd:statef')) block.setPermutation(block.permutation.withState("bd:state", 4));
        dimension.playSound("item.bone_meal.use", block.center());
        dimension.spawnParticle("minecraft:crop_growth_emitter", block.center())
      }
    }
  }
};

const box = {
  onTick({ block, player, dimension }) {
    const blockType = block.typeId;
    const prefix = "bd:";
    const pizzaTypes = [
      "pepperoni",
      "margherita",
      "neapolitan",
      "mushroom",
      "4cheeses",
      "marinara",
      "newyorker",
      "hawaiian",
      "fugazza",
      "4seasons"
    ];

    pizzaTypes.forEach((pizzaType) => {
      for (let i = 1; i <= 7; i++) {
        const blockId = `${prefix}${pizzaType}_pizza_box_${i}`;
        if (blockType === blockId) {
          const direction = block.permutation.getState("minecraft:cardinal_direction");
          const slices = i;
          block.setPermutation(BlockPermutation.resolve(`${prefix}${pizzaType}_pizza_box`, {
            "minecraft:cardinal_direction": direction,
            "bd:slices": slices
          }));
        }
      }
    });
  }
}

const boxPizza = {
  onPlayerInteract({block, player, dimension}) {

    const stateSlices = block.permutation.getState("bd:slices");
    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;
    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
    const pizzas = {
      "bd:pepperoni_pizza_shovel": "bd:pepperoni_pizza_box",
      "bd:margherita_pizza_shovel": "bd:margherita_pizza_box",
      "bd:neapolitan_pizza_shovel": "bd:neapolitan_pizza_box",
      "bd:mushroom_pizza_shovel": "bd:mushroom_pizza_box",
      "bd:4cheeses_pizza_shovel": "bd:4cheeses_pizza_box",
      "bd:marinara_pizza_shovel": "bd:marinara_pizza_box",
      "bd:newyorker_pizza_shovel": "bd:newyorker_pizza_box",
      "bd:hawaiian_pizza_shovel": "bd:hawaiian_pizza_box",
      "bd:fugazza_pizza_shovel": "bd:fugazza_pizza_box",
      "bd:4seasons_pizza_shovel": "bd:4seasons_pizza_box"
    };

    if (block.permutation.getState("bd:state") == 0)
      block.setPermutation(block.permutation.withState("bd:state", 1));
    else if (mainhand.hasItem() && mainhand.typeId == "bd:knife" && block.typeId != "bd:pizza_box") {
      const pizzaSlices = {
        "bd:pepperoni_pizza_box": "bd:pepperoni_pizza_slice",
        "bd:margherita_pizza_box": "bd:margherita_pizza_slice",
        "bd:neapolitan_pizza_box": "bd:neapolitan_pizza_slice",
        "bd:mushroom_pizza_box": "bd:mushroom_pizza_slice",
        "bd:4cheeses_pizza_box": "bd:4cheeses_pizza_slice",
        "bd:marinara_pizza_box": "bd:marinara_pizza_slice",
        "bd:newyorker_pizza_box": "bd:newyorker_pizza_slice",
        "bd:hawaiian_pizza_box": "bd:hawaiian_pizza_slice",
        "bd:fugazza_pizza_box": "bd:fugazza_pizza_slice"
      };
      const seasons = {
        0: "bd:summer_pizza_slice",
        1: "bd:summer_pizza_slice",
        2: "bd:spring_pizza_slice",
        3: "bd:spring_pizza_slice",
        4: "bd:winter_pizza_slice",
        5: "bd:winter_pizza_slice",
        6: "bd:autumn_pizza_slice",
        7: "bd:autumn_pizza_slice"
      };
      dimension.playSound("block.beehive.shear", block.center());
      if (pizzaSlices[block.typeId] && block.permutation.getState("bd:state") == 1) {
        dimension.spawnItem(new ItemStack(pizzaSlices[block.typeId], 1), block.center());
      }
      if (block.typeId == "bd:4seasons_pizza_box") {
        dimension.spawnItem(new ItemStack(seasons[stateSlices], 1), block.center());
      }
      if (block.permutation.getState("bd:slices") == 7) {
        const direction = block.permutation.getState("minecraft:cardinal_direction");
        const blockCardinal = BlockPermutation.resolve("bd:pizza_box", {
          "minecraft:cardinal_direction": direction,
          "bd:state": 1
        });
        block.setPermutation(blockCardinal);
        return;
      }
      block.setPermutation(block.permutation.withState("bd:slices", stateSlices + 1));
    }
    else if (mainhand.hasItem() && pizzas[mainhand.typeId] && block.typeId == "bd:pizza_box" && block.permutation.getState("bd:state") == 1) {
      const pizzaBlock = pizzas[mainhand.typeId];
      if (pizzaBlock) {
        dimension.playSound("hit.cloth", block.center());
        if (player.getGameMode() === GameMode.survival) mainhand.setItem(new ItemStack("bd:pizza_shovel"));
        const direction = block.permutation.getState("minecraft:cardinal_direction");
        const blockCardinal = BlockPermutation.resolve(pizzaBlock, {
          "minecraft:cardinal_direction": direction,
          "bd:state": 1
        });
        block.setPermutation(blockCardinal);
      }
      return;
    }
    else if (block.permutation.getState("bd:state") == 1)
      block.setPermutation(block.permutation.withState("bd:state", 0));
  }
};

const board = {
  onPlayerInteract({block, player, dimension}) {
    const cut = block.permutation.getState("bd:cut");
    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;
    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);

    if (mainhand.hasItem() || !mainhand.hasItem()) {
      if (block.permutation.getState("bd:item") == 6) {
        block.setPermutation(block.permutation.withState("bd:item", 0));
        dimension.playSound("block.itemframe.remove_item", block.center());
        if (player.getGameMode() === GameMode.survival)
          dimension.spawnItem(new ItemStack("bd:knife", 1), block.center());
        return;
      }
    }
    if (mainhand.hasItem() && block.permutation.getState("bd:item") == 0) {
      if (mainhand.typeId == "bd:cheese" && block.permutation.getState("bd:item") == 0)
        block.setPermutation(block.permutation.withState("bd:item", 1));
      if (mainhand.typeId == "bd:pepperoni" && block.permutation.getState("bd:item") == 0)
        block.setPermutation(block.permutation.withState("bd:item", 2));
      if (mainhand.typeId == "bd:pineapple" && block.permutation.getState("bd:item") == 0)
        block.setPermutation(block.permutation.withState("bd:item", 3));
      if (mainhand.typeId == "bd:onion" && block.permutation.getState("bd:item") == 0)
        block.setPermutation(block.permutation.withState("bd:item", 4));
      if (mainhand.typeId == "minecraft:porkchop" && block.permutation.getState("bd:item") == 0)
        block.setPermutation(block.permutation.withState("bd:item", 5));
      if (mainhand.typeId == "bd:knife" && block.permutation.getState("bd:item") == 0) {
        block.setPermutation(block.permutation.withState("bd:item", 6));
        if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
        else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
        dimension.playSound("dig.wood", block.center());
        return;
      }
      if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
      else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
      if (block.permutation.getState("bd:item") > 0) dimension.playSound("hit.cloth", block.center());
    }
      
    if (mainhand.hasItem()) {
      if (mainhand.typeId == "bd:knife") {
        if (block.permutation.getState("bd:item") == 1) {
          if (cut == 3) {  
            dimension.playSound("block.beehive.shear", block.center());
            dimension.spawnItem(new ItemStack("bd:cheese_slice", 1), block.center());
            block.setPermutation(block.permutation.withState("bd:item", 0));
            block.setPermutation(block.permutation.withState("bd:cut", 0));
            return;
          }
          dimension.playSound("block.beehive.shear", block.center());
          dimension.spawnItem(new ItemStack("bd:cheese_slice", 1), block.center());
          block.setPermutation(block.permutation.withState("bd:cut", cut + 1));
          return;
        }
        if (block.permutation.getState("bd:item") == 2) {
          if (cut == 4) {
            dimension.spawnItem(new ItemStack("bd:pepperoni_slice", 1), block.center());
            dimension.playSound("block.beehive.shear", block.center());
            block.setPermutation(block.permutation.withState("bd:item", 0));
            block.setPermutation(block.permutation.withState("bd:cut", 0));
            return;
          }
          dimension.spawnItem(new ItemStack("bd:pepperoni_slice", 1), block.center());
          dimension.playSound("block.beehive.shear", block.center());
          block.setPermutation(block.permutation.withState("bd:cut", cut + 1));
          return;
        }
        if (block.permutation.getState("bd:item") == 3) {
          if (cut == 0) {
            dimension.spawnItem(new ItemStack("bd:pineapple_leaf", 1), block.center());
            dimension.playSound("block.beehive.shear", block.center());
          }
          if (cut == 5) {
            dimension.spawnItem(new ItemStack("bd:pineapple_slice", 1), block.center());
            dimension.playSound("block.beehive.shear", block.center());
            block.setPermutation(block.permutation.withState("bd:item", 0));
            block.setPermutation(block.permutation.withState("bd:cut", 0));
            return;
          }
          if (cut > 1) {
            dimension.spawnItem(new ItemStack("bd:pineapple_slice", 1), block.center());
          }
          dimension.playSound("block.beehive.shear", block.center());
          block.setPermutation(block.permutation.withState("bd:cut", cut + 1));
          return;
        }
        if (block.permutation.getState("bd:item") == 4) {
          if (cut == 5) {
            dimension.spawnItem(new ItemStack("bd:chopped_onion", 1), block.center());
            dimension.playSound("block.beehive.shear", block.center());
            block.setPermutation(block.permutation.withState("bd:item", 0));
            block.setPermutation(block.permutation.withState("bd:cut", 0));
            return;
          }
          if (cut > 1) dimension.spawnItem(new ItemStack("bd:chopped_onion", 1), block.center());
          dimension.playSound("block.beehive.shear", block.center());
          block.setPermutation(block.permutation.withState("bd:cut", cut + 1));
          return;
        }
        if (block.permutation.getState("bd:item") == 5) {
          if (cut == 3) {
            dimension.spawnItem(new ItemStack("bd:ham", 1), block.center());
            dimension.playSound("block.beehive.shear", block.center());
            block.setPermutation(block.permutation.withState("bd:item", 0));
            block.setPermutation(block.permutation.withState("bd:cut", 0));
            return;
          }
          dimension.spawnItem(new ItemStack("bd:ham", 1), block.center());
          dimension.playSound("block.beehive.shear", block.center());
          block.setPermutation(block.permutation.withState("bd:cut", cut + 1));
          return;
        }
      }
    }
  }
};

const dish = {
  onPlayerInteract({block, player, dimension}) {
    const pizzaSliceState = block.permutation.getState("bd:slice");
    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;
    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
    const pizza = {
      "bd:pepperoni_pizza_shovel": 1,
      "bd:margherita_pizza_shovel": 2,
      "bd:neapolitan_pizza_shovel": 3,
      "bd:mushroom_pizza_shovel": 4,
      "bd:4cheeses_pizza_shovel": 5,
      "bd:marinara_pizza_shovel": 6,
      "bd:newyorker_pizza_shovel": 7,
      "bd:hawaiian_pizza_shovel": 8,
      "bd:fugazza_pizza_shovel": 9,
      "bd:4seasons_pizza_shovel": 10
    };
    const pizzaSlice = {
      1: "bd:pepperoni_pizza_slice",
      2: "bd:margherita_pizza_slice",
      3: "bd:neapolitan_pizza_slice",
      4: "bd:mushroom_pizza_slice",
      5: "bd:4cheeses_pizza_slice",
      6: "bd:marinara_pizza_slice",
      7: "bd:newyorker_pizza_slice",
      8: "bd:hawaiian_pizza_slice",
      9: "bd:fugazza_pizza_slice",
      10: "bd:4seasons_pizza_slice"
    };
    const seasons = {
      0: "bd:summer_pizza_slice",
      1: "bd:summer_pizza_slice",
      2: "bd:spring_pizza_slice",
      3: "bd:spring_pizza_slice",
      4: "bd:winter_pizza_slice",
      5: "bd:winter_pizza_slice",
      6: "bd:autumn_pizza_slice",
      7: "bd:autumn_pizza_slice"
    };
    if (mainhand.hasItem()) {
      const pizzaState = pizza[mainhand.typeId];
      if (pizzaState && block.permutation.getState("bd:state") == 0) {
        block.setPermutation(block.permutation.withState("bd:state", pizzaState));
        if (player.getGameMode() === GameMode.survival) mainhand.setItem(new ItemStack("bd:pizza_shovel"));
        dimension.playSound("hit.cloth", block.center());
        return;
      }
      if (mainhand.typeId == "bd:knife") {
        const currentBlock = block.permutation.getState("bd:state");
        const currentBlockSlice = block.permutation.getState("bd:slice");
        const pizzaSliceId = pizzaSlice[currentBlock];
        const pizzaTenSliceId = seasons[currentBlockSlice];
        if (currentBlockSlice < 7 && currentBlock > 0 && currentBlock < 10) {
          block.setPermutation(block.permutation.withState("bd:slice", pizzaSliceState + 1));
          dimension.playSound("block.beehive.shear", block.center());
          dimension.spawnItem(new ItemStack(pizzaSliceId, 1), block.center());
          return;
        } else if (block.permutation.getState("bd:slice") == 7 && currentBlock < 10) {
            dimension.playSound("block.beehive.shear", block.center());
            dimension.spawnItem(new ItemStack(pizzaSliceId, 1), block.center());
            block.setPermutation(block.permutation.withState("bd:state", 0));
            block.setPermutation(block.permutation.withState("bd:slice", 0));
            return;
        }
        if (currentBlockSlice < 7 && currentBlock == 10) {
          block.setPermutation(block.permutation.withState("bd:slice", pizzaSliceState + 1));
          dimension.playSound("block.beehive.shear", block.center());
          dimension.spawnItem(new ItemStack(pizzaTenSliceId, 1), block.center());
          return;
        } else if (currentBlockSlice == 7) {
            dimension.playSound("block.beehive.shear", block.center());
            dimension.spawnItem(new ItemStack(pizzaTenSliceId, 1), block.center());
            block.setPermutation(block.permutation.withState("bd:state", 0));
            block.setPermutation(block.permutation.withState("bd:slice", 0));
        }
      }
    }
  }
};

const glass = {
  onPlayerInteract({block, player, dimension}) {
    const sauce = block.permutation.getState("bd:sauce");
    
    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;
    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
    
    if (mainhand.hasItem()) {
      if (mainhand.typeId == "bd:tomato_sauce_bottle" && block.permutation.getState("bd:sauce") < 4) {
        if (!player.getComponent("inventory").container.addItem(new ItemStack("minecraft:glass_bottle", 1))) {
        } else dimension.spawnItem(new ItemStack("minecraft:glass_bottle", 1), block.center());
        if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
        else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
        block.setPermutation(block.permutation.withState("bd:sauce", sauce + 1));
        dimension.playSound("bucket.empty_water", block.center());
        return;
      }
      if (mainhand.typeId == "minecraft:glass_bottle" && block.permutation.getState("bd:sauce") > 0) {
        if (!player.getComponent("inventory").container.addItem(new ItemStack("bd:tomato_sauce_bottle", 1))) {
        } else dimension.spawnItem(new ItemStack("bd:tomato_sauce_bottle", 1), block.center());
        if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
        else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
        block.setPermutation(block.permutation.withState("bd:sauce", sauce - 1));
        dimension.playSound("bucket.empty_water", block.center());
      }
    }
  }
};

const pot = {
  onPlayerInteract({block, player, dimension}) {
    const allBlockStates = block.permutation.getAllStates();
    if (Object.keys(allBlockStates).length === 0) return;
    const allStateZero =Object.values(allBlockStates).filter(value => typeof value === 'number').every(value => value === 0);
    
    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;
    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
    
    if (mainhand.hasItem()) {
      if (mainhand.typeId == "minecraft:water_bucket" && allStateZero) {
        if (player.getGameMode() === GameMode.survival) mainhand.setItem(new ItemStack("minecraft:bucket"));
        block.setPermutation(block.permutation.withState("bd:salt", 1));
        dimension.playSound("bucket.empty_water", block.center());
        dimension.playSound("extinguish.candle", block.center());
        return;
      }
      if (mainhand.typeId == "minecraft:milk_bucket" && allStateZero) {
        if (player.getGameMode() === GameMode.survival) mainhand.setItem(new ItemStack("minecraft:bucket"));
        block.setPermutation(block.permutation.withState("bd:cheese", 1));
        dimension.playSound("bucket.empty_water", block.center());
        return;
      } else if (mainhand.typeId == "bd:salt" && block.permutation.getState('bd:cheese') == 1) {
          if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
          else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
          block.setPermutation(block.permutation.withState("bd:cheese", 2));
          dimension.playSound("extinguish.candle", block.center());
      }
    }
    if (mainhand.hasItem() || !mainhand.hasItem()) {
      if (block.permutation.getState('bd:salt') == 2) {
        block.setPermutation(block.permutation.withState("bd:salt", 0));
        dimension.spawnItem(new ItemStack("bd:salt", 2), block.center());
        dimension.playSound("block.itemframe.remove_item", block.center());
        return;
      }
      if (block.permutation.getState('bd:cheese') == 3) {
        block.setPermutation(block.permutation.withState("bd:cheese", 0));
        dimension.spawnItem(new ItemStack("bd:cheese", 1), block.center());
        dimension.playSound("block.itemframe.remove_item", block.center());
        return;
      }
    }
  },
  onTick({block, dimension}) {
    if (block.permutation.getState("bd:salt") == 1) {
      block.setPermutation(block.permutation.withState("bd:salt", 2));
      dimension.playSound("extinguish.candle", block.center());
      return;
    }
    if (block.permutation.getState("bd:cheese") == 2) {
      block.setPermutation(block.permutation.withState("bd:cheese", 3));
      dimension.playSound("extinguish.candle", block.center());
    }
  }
};

const pestle = {
  onPlayerInteract({block, player, dimension}) {
    //Detectar si todos los estados son 0
    const allBlockStates = block.permutation.getAllStates();
    if (Object.keys(allBlockStates).length === 0) return;
    const allStateZero =Object.values(allBlockStates).filter(value => typeof value === 'number').every(value => value === 0);

    //Variable
    const sauce = block.permutation.getState("bd:sauce");

    //Detectar que item tiene el jugador en la mano
    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;
    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);

    if (mainhand.hasItem() || !mainhand.hasItem()) {
      if (block.permutation.getState("bd:mortar") == 1) {
        block.setPermutation(block.permutation.withState("bd:mortar", 0));
        dimension.playSound("block.itemframe.remove_item", block.center());
        if (player.getGameMode() === GameMode.survival) dimension.spawnItem(new ItemStack("bd:mortar", 1), block.center());
        return;
      }
    }

    if (block.permutation.getState("bd:sauce") <= 3 && mainhand.hasItem() && mainhand.typeId == "bd:tomato") {
      block.setPermutation(block.permutation.withState("bd:sauce", sauce + 1));
      if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
      else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
      return;
    }
    if (block.permutation.getState("bd:sauce") >= 4 && block.permutation.getState("bd:sauce") <= 5 && mainhand.hasItem() && mainhand.typeId == "bd:basil_leaf") {
      block.setPermutation(block.permutation.withState("bd:sauce", sauce + 1));
      if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
      else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
      return;
    }
    if (mainhand.hasItem() && mainhand.typeId == "bd:mortar") {
      if (block.permutation.getState("bd:sauce") == 6) {
        block.setPermutation(block.permutation.withState("bd:sauce", sauce + 1));
        dimension.playSound("block.beehive.drip", block.center());
        return;
      }
      if (allStateZero) {
        if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
        dimension.playSound("armor.equip_generic", block.center())
        block.setPermutation(block.permutation.withState("bd:mortar", 1));
      }
    }
    if (block.permutation.getState("bd:sauce") == 7 && mainhand.hasItem() && mainhand.typeId == "bd:salt") {
      dimension.playSound("block.beehive.drip", block.center());
      block.setPermutation(block.permutation.withState("bd:sauce", sauce + 1));
      if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
      else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
      return;
    }
    if (block.permutation.getState("bd:sauce") == 8 && mainhand.hasItem() && mainhand.typeId == "minecraft:glass_bottle") {

      block.setPermutation(block.permutation.withState("bd:sauce", 0));

      if (!player.getComponent("inventory").container.addItem(new ItemStack("bd:tomato_sauce_bottle", 1))) {
      } else dimension.spawnItem(new ItemStack("bd:tomato_sauce_bottle", 1), block.center());

      dimension.playSound("copper.wax.on", block.center());
      if (player.getGameMode() === GameMode.survival && mainhand.amount > 1) mainhand.amount--;
      else if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
    }
  }
};

const bar = {
  onPlayerInteract({block, player, dimension}) {
    if (block.permutation.getState("bd:item") == 1) {
      block.setPermutation(block.permutation.withState("bd:item", 0));
      dimension.playSound("block.itemframe.remove_item", block.center());
      if (player.getGameMode() === GameMode.survival) dimension.spawnItem(new ItemStack("bd:pizza_shovel", 1), block.center());
      return;
    }
    const equippable = player.getComponent("minecraft:equippable");
    const mainhand = equippable.getEquipmentSlot (EquipmentSlot.Mainhand);

    if (!mainhand.hasItem() || mainhand.typeId !== "bd:pizza_shovel") {
      return;
    } else if (block.permutation.getState("bd:item") == 0) {
      block.setPermutation(block.permutation.withState("bd:item", 1));
      if (player.getGameMode() === GameMode.survival) mainhand.setItem(undefined);
      dimension.playSound("armor.equip_generic", block.center())
    return;
    }
  }
};

const pizzaCook = {
  onTick({block, dimension}){
    if (block.permutation.getState("bd:state") == 0) {
      block.setPermutation(block.permutation.withState("bd:state", 1));
      dimension.playSound("extinguish.candle", block.center());
    }
  },
  beforeOnPlayerPlace({block, dimension, player}){
    if (player.getGameMode() === GameMode.survival) {
        player.runCommandAsync("replaceitem entity @s slot.weapon.mainhand 0 bd:pizza_shovel");
    }
    dimension.playSound("extinguish.candle", block.center());
  },
  onPlayerInteract({block, player}) {
    const rawPizza = {
      "bd:pepperoni_pizza": "bd:raw_pepperoni_pizza_shovel",
      "bd:margherita_pizza": "bd:raw_margherita_pizza_shovel",
      "bd:neapolitan_pizza": "bd:raw_neapolitan_pizza_shovel",
      "bd:mushroom_pizza": "bd:raw_mushroom_pizza_shovel",
      "bd:4cheeses_pizza": "bd:raw_4cheeses_pizza_shovel",
      "bd:marinara_pizza": "bd:raw_marinara_pizza_shovel",
      "bd:newyorker_pizza": "bd:raw_newyorker_pizza_shovel",
      "bd:hawaiian_pizza": "bd:raw_hawaiian_pizza_shovel",
      "bd:fugazza_pizza": "bd:raw_fugazza_pizza_shovel",
      "bd:4seasons_pizza": "bd:raw_4seasons_pizza_shovel"
    };
    const cookedPizza = {
      "bd:pepperoni_pizza": "bd:pepperoni_pizza_shovel",
      "bd:margherita_pizza": "bd:margherita_pizza_shovel",
      "bd:neapolitan_pizza": "bd:neapolitan_pizza_shovel",
      "bd:mushroom_pizza": "bd:mushroom_pizza_shovel",
      "bd:4cheeses_pizza": "bd:4cheeses_pizza_shovel",
      "bd:marinara_pizza": "bd:marinara_pizza_shovel",
      "bd:newyorker_pizza": "bd:newyorker_pizza_shovel",
      "bd:hawaiian_pizza": "bd:hawaiian_pizza_shovel",
      "bd:fugazza_pizza": "bd:fugazza_pizza_shovel",
      "bd:4seasons_pizza": "bd:4seasons_pizza_shovel"
    };
    const rawPizzaType = rawPizza[block.typeId];
    const cookedPizzaType = cookedPizza[block.typeId];

    const equippable = player?.getComponent("minecraft:equippable");
    if (!equippable) return;

    const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
    if (!mainhand.hasItem() || mainhand.typeId !== "bd:pizza_shovel") return;

    if (rawPizzaType && block.permutation.getState("bd:state") == 0) {
      mainhand.setItem(new ItemStack(rawPizzaType));
    } else if (cookedPizzaType && block.permutation.getState("bd:state") == 1) {
        mainhand.setItem(new ItemStack(cookedPizzaType));
    }
    block.setType("minecraft:air");
    player.playSound("random.pop");
  }
};

world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
  blockComponentRegistry.registerCustomComponent("bd:bar", bar);
  blockComponentRegistry.registerCustomComponent("bd:cook", pizzaCook);
  blockComponentRegistry.registerCustomComponent("bd:pestle", pestle);
  blockComponentRegistry.registerCustomComponent("bd:dish", dish);
  blockComponentRegistry.registerCustomComponent("bd:pot", pot);
  blockComponentRegistry.registerCustomComponent("bd:glass", glass);
  blockComponentRegistry.registerCustomComponent("bd:board", board);
  blockComponentRegistry.registerCustomComponent("bd:pizza_box", boxPizza);
  blockComponentRegistry.registerCustomComponent("bd:box", box);
  blockComponentRegistry.registerCustomComponent("bd:crop", crop);
});

world.afterEvents.playerPlaceBlock.subscribe((event) => {
    const eb = event.block;
    const northBlock = {
      left: eb.dimension.getBlock({ x: eb.location.x + 1, y: eb.location.y, z: eb.location.z }), 
      right: eb.dimension.getBlock({ x: eb.location.x - 1, y: eb.location.y, z: eb.location.z }), 
    };
    const southBlock = {
      left: eb.dimension.getBlock({ x: eb.location.x - 1, y: eb.location.y, z: eb.location.z }), 
      right: eb.dimension.getBlock({ x: eb.location.x + 1, y: eb.location.y, z: eb.location.z }), 
    };
    const eastBlock = {
      left: eb.dimension.getBlock({ x: eb.location.x, y: eb.location.y, z: eb.location.z - 1}), 
      right: eb.dimension.getBlock({ x: eb.location.x, y: eb.location.y, z: eb.location.z + 1}), 
    };
    const cardinal = {
      northLeft: northBlock.left.typeId === "bd:bar" && northBlock.left.permutation.getState('minecraft:cardinal_direction') === 'north',
      northRight: northBlock.right.typeId === "bd:bar" && northBlock.right.permutation.getState('minecraft:cardinal_direction') === 'north',
  };
    const initState = {
      north: northBlock.left.permutation.getState('bd:conect') === 0 || northBlock.right.permutation.getState('bd:conect') === 0,
    };
    const statePosition = {
      north: northBlock.left.permutation.getState('bd:conect') === 1 || northBlock.right.permutation.getState('bd:conect') === 2,
      northOne: northBlock.left.permutation.getState('bd:conect') === 1 && northBlock.right.permutation.getState('bd:conect') === 0,
      northTwo: northBlock.left.permutation.getState('bd:conect') === 0 && northBlock.right.permutation.getState('bd:conect') === 2,
    };
    if (eb.typeId == "bd:bar" && eb.permutation.getState('minecraft:cardinal_direction') === 'north') {
      //junta 2 bloques de estado 0 iniciales
      if (cardinal.northLeft && initState.north) {
        eb.setPermutation(eb.permutation.withState('bd:conect', 1));
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 2));
      } else if (cardinal.northRight && initState.north) {
          eb.setPermutation(eb.permutation.withState('bd:conect', 2));
          northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 1));
      }
      //junta bloques adyacentes, se ejecuta si el de arriba es ejecutado
      if (cardinal.northLeft && statePosition.north) {
        eb.setPermutation(eb.permutation.withState('bd:conect', 1));
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 3));
      } else if (cardinal.northRight && statePosition.north) {
        eb.setPermutation(eb.permutation.withState('bd:conect', 2));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 3));
      }
      //detecta si 1 bloque es colocado en el centro de 2 bloques en estado 0
      if (cardinal.northLeft && cardinal.northRight && initState.north) {
        eb.setPermutation(eb.permutation.withState('bd:conect', 3));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 1));
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 2));
      }
      //detecta si 1 bloque es colocado en el centro de 2 bloques en estados 1 y 2
      if (cardinal.northLeft && cardinal.northRight && statePosition.north) {
        eb.setPermutation(eb.permutation.withState('bd:conect', 3));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 3));
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 3));
      }
      if (cardinal.northRight && cardinal.northLeft && statePosition.northOne) {
        eb.setPermutation(eb.permutation.withState('bd:conect', 3));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 1));
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 3));
      } else if (cardinal.northRight && cardinal.northLeft && statePosition.northTwo) {
        eb.setPermutation(eb.permutation.withState('bd:conect', 3));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 3));
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 2));
      }
    }
  });
  world.afterEvents.playerBreakBlock.subscribe((event) => {
  
    eb = event.block;
    const northBlock = {
      left: eb.dimension.getBlock({ x: eb.location.x + 1, y: eb.location.y, z: eb.location.z }), 
      right: eb.dimension.getBlock({ x: eb.location.x - 1, y: eb.location.y, z: eb.location.z }), 
    };
    const cardinal = {
      northLeft: northBlock.left.typeId === "bd:bar" && northBlock.left.permutation.getState('minecraft:cardinal_direction') === 'north',
      northRight: northBlock.right.typeId === "bd:bar" && northBlock.right.permutation.getState('minecraft:cardinal_direction') === 'north',
  };
    const statePosition = {
      north: northBlock.left.permutation.getState('bd:conect') === 2 || northBlock.right.permutation.getState('bd:conect') === 1,
      northSide: northBlock.left.permutation.getState('bd:conect') === 3 || northBlock.right.permutation.getState('bd:conect') === 3,
      northStateOne: northBlock.left.permutation.getState('bd:conect') === 2 && northBlock.right.permutation.getState('bd:conect') === 1,
      northStateTwo: northBlock.left.permutation.getState('bd:conect') === 3 && northBlock.right.permutation.getState('bd:conect') === 3,
      northStateThree: northBlock.left.permutation.getState('bd:conect') === 2 && northBlock.right.permutation.getState('bd:conect') === 3,
      northStateFour: northBlock.left.permutation.getState('bd:conect') === 3 && northBlock.right.permutation.getState('bd:conect') === 1,
    };
    if (event.brokenBlockPermutation.type.id === "bd:bar" && event.brokenBlockPermutation.getState('minecraft:cardinal_direction') === 'north') {
      //devuelve al estado 0 si 2 bloques estan juntos adyacentemente
      if (cardinal.northLeft && statePosition.north) {
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 0));
      }
      else if (cardinal.northRight && statePosition.north) {
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 0));
      }
      else if (cardinal.northLeft && statePosition.northSide) {
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 1));
      }
      else if (cardinal.northRight && statePosition.northSide) {
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 2));
      }
      //devuelve al estado 0 a 2 bloques si el bloque roto es del cemtro
      if (cardinal.northLeft && cardinal.northRight && statePosition.northStateOne) {
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 0));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 0));
      }
      if (cardinal.northLeft && cardinal.northRight && statePosition.northStateTwo) {
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 1));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 2));
      }
      if (cardinal.northLeft && cardinal.northRight && statePosition.northStateThree) {
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 0));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 2));
      }
      else if (cardinal.northLeft && cardinal.northRight && statePosition.northStateFour) {
        northBlock.left.setPermutation(northBlock.left.permutation.withState('bd:conect', 1));
        northBlock.right.setPermutation(northBlock.right.permutation.withState('bd:conect', 0));
      }
   }
});